@@ -1,47 +1,58 @@
 {-# LANGUAGE CPP, DeriveDataTypeable, FlexibleContexts, MultiParamTypeClasses #-}
 {-# LANGUAGE OverloadedStrings, StandaloneDeriving                            #-}
-{-# OPTIONS_GHC -Wall -fno-warn-orphans #-}
 module Web.Authenticate.OAuth
     ( -- * Data types
       OAuth, def, newOAuth, oauthServerName, oauthRequestUri, oauthAccessTokenUri,
       oauthAuthorizeUri, oauthSignatureMethod, oauthConsumerKey,
       oauthConsumerSecret, oauthCallback, oauthRealm, oauthVersion,
       OAuthVersion(..), SignMethod(..), Credential(..), OAuthException(..),
+      -- ** Access token request
+      AccessTokenRequest,
+      defaultAccessTokenRequest,
+      accessTokenAddAuth,
+      accessTokenRequestHook,
+      accessTokenOAuth,
+      accessTokenTemporaryCredential,
+      accessTokenManager,
       -- * Operations for credentials
       newCredential, emptyCredential, insert, delete, inserts, injectVerifier,
       -- * Signature
       signOAuth, genSign,
       -- * Url & operation for authentication
-      authorizeUrl, authorizeUrl', getAccessToken, getTemporaryCredential,
-      getTokenCredential, getTemporaryCredentialWithScope,
-      getAccessTokenProxy, getTemporaryCredentialProxy,
+      -- ** Temporary credentials
+      getTemporaryCredential, getTemporaryCredentialWithScope,
+      getTemporaryCredentialProxy, getTemporaryCredential',
+      -- ** Authorization URL
+      authorizeUrl, authorizeUrl',
+      -- ** Attaching auth to requests
+      addAuthBody,
+      -- ** Finishing authentication
+      getAccessToken,
+      getAccessTokenProxy,
+      getTokenCredential,
       getTokenCredentialProxy,
-      getAccessToken', getTemporaryCredential',
+      getAccessToken',
+      getAccessTokenWith,
       -- * Utility Methods
       paramEncode, addScope, addMaybeProxy
     ) where
-import           Blaze.ByteString.Builder     (toByteString, Builder)
-import           Codec.Crypto.RSA             (ha_SHA1, rsassa_pkcs1_v1_5_sign)
+
+import           Blaze.ByteString.Builder     (toByteString)
 import           Control.Exception
 import           Control.Monad
 import           Control.Monad.IO.Class       (MonadIO, liftIO)
-import           Control.Monad.Trans.Control
-import           Control.Monad.Trans.Resource
 import           Crypto.Types.PubKey.RSA      (PrivateKey (..), PublicKey (..))
 import           Data.ByteString.Base64
 import qualified Data.ByteString.Char8        as BS
 import qualified Data.ByteString.Lazy.Char8   as BSL
 import           Data.Char
-import           Data.Conduit                 (Source, ($$), ($=))
-import           Data.Conduit.Blaze           (builderToByteString)
-import qualified Data.Conduit.List            as CL
 import           Data.Default
 import           Data.Digest.Pure.SHA
 import qualified Data.IORef                   as I
-import           Data.List                    (sortBy)
+import           Data.List                    (sort)
 import           Data.Maybe
 import           Data.Time
-import           Network.HTTP.Conduit
+import           Network.HTTP.Client
 import           Network.HTTP.Types           (SimpleQuery, parseSimpleQuery)
 import           Network.HTTP.Types           (Header)
 import           Network.HTTP.Types           (renderSimpleQuery, status200)
@@ -52,6 +63,12 @@ import Data.Data hiding (Proxy (..))
 #else
 import Data.Data
 #endif
+import Codec.Crypto.RSA (rsassa_pkcs1_v1_5_sign, hashSHA1)
+
+
+----------------------------------------------------------------------
+-- Data types
+
 
 -- | Data type for OAuth client (consumer).
 --
@@ -66,7 +83,7 @@ data OAuth = OAuth { oauthServerName      :: String -- ^ Service name (default:
                    --   or 'getTemporaryCredential'; otherwise you can just leave this empty.
                    , oauthAccessTokenUri  :: String
                    -- ^ Uri to obtain access token (default: @\"\"@).
-                   --   You MUST specify if you use 'getAcessToken' or 'getAccessToken'';
+                   --   You MUST specify if you use 'getAcessToken' or 'getAccessToken'' or 'getAccessTokenWith';
                    --   otherwise you can just leave this empty.
                    , oauthAuthorizeUri    :: String
                    -- ^ Uri to authorize (default: @\"\"@).
@@ -84,11 +101,13 @@ data OAuth = OAuth { oauthServerName      :: String -- ^ Service name (default:
                    -- ^ Optional authorization realm (default: @Nothing@)
                    , oauthVersion         :: OAuthVersion
                    -- ^ OAuth spec version (default: 'OAuth10a')
-                   } deriving (Show, Eq, Ord, Read, Data, Typeable)
+                   } deriving (Show, Eq, Read, Data, Typeable)
+
 
 data OAuthVersion = OAuth10     -- ^ OAuth protocol ver 1.0 (no oauth_verifier; differs from RFC 5849).
                   | OAuth10a    -- ^ OAuth protocol ver 1.0a. This corresponds to community's 1.0a spec and RFC 5849.
-                    deriving (Show, Eq, Ord, Data, Typeable, Read)
+                    deriving (Show, Eq, Enum, Ord, Data, Typeable, Read)
+
 
 -- | Default value for OAuth datatype.
 -- You must specify at least oauthServerName, URIs and Tokens.
@@ -108,21 +127,75 @@ newOAuth = OAuth { oauthSignatureMethod = HMACSHA1
 instance Default OAuth where
   def = newOAuth
 
+
 -- | Data type for signature method.
 data SignMethod = PLAINTEXT
                 | HMACSHA1
                 | RSASHA1 PrivateKey
-                  deriving (Show, Eq, Ord, Read, Data, Typeable)
-deriving instance Ord PrivateKey
-deriving instance Ord PublicKey
+                  deriving (Show, Eq, Read, Data, Typeable)
+
+
+data OAuthException = OAuthException String
+                      deriving (Show, Eq, Data, Typeable)
+
+instance Exception OAuthException
+
+
+-- | Data type for getAccessTokenWith method.
+--
+-- You can create values of this type using 'defaultAccessTokenRequest'.
+--
+-- Since 1.5.1
+data AccessTokenRequest = AccessTokenRequest {
+    accessTokenAddAuth :: (BS.ByteString -> Credential -> Request -> Request)
+    -- ^ add auth hook.
+    --
+    -- Default: addAuthHeader
+    --
+    -- Since 1.5.1
+  , accessTokenRequestHook :: (Request -> Request)
+    -- ^ Request Hook.
+    --
+    -- Default: @id@
+    --
+    -- Since 1.5.1
+  , accessTokenOAuth :: OAuth
+    -- ^ OAuth Application
+    --
+    -- Since 1.5.1
+  , accessTokenTemporaryCredential :: Credential
+    -- ^ Temporary Credential (with oauth_verifier if >= 1.0a)
+    --
+    -- Since 1.5.1
+  , accessTokenManager :: Manager
+    -- ^ Manager
+    --
+    -- Since 1.5.1
+  }
+
+-- | Create a value of type 'AccessTokenRequest' with default values filled in.
+--
+-- Note that this is a settings type. More information on usage can be found
+-- at: <http://www.yesodweb.com/book/settings-types>.
+--
+-- Since 1.5.1
+defaultAccessTokenRequest :: OAuth -> Credential -> Manager -> AccessTokenRequest
+defaultAccessTokenRequest oauth cred man = AccessTokenRequest
+    { accessTokenAddAuth = addAuthHeader
+    , accessTokenRequestHook = id
+    , accessTokenOAuth = oauth
+    , accessTokenTemporaryCredential = cred
+    , accessTokenManager = man
+    }
+
+----------------------------------------------------------------------
+-- Credentials
+
 
 -- | Data type for redential.
 data Credential = Credential { unCredential :: [(BS.ByteString, BS.ByteString)] }
                   deriving (Show, Eq, Ord, Read, Data, Typeable)
 
--- | Empty credential.
-emptyCredential :: Credential
-emptyCredential = Credential []
 
 -- | Convenient function to create 'Credential' with OAuth Token and Token Secret.
 newCredential :: BS.ByteString -- ^ value for oauth_token
@@ -130,58 +203,112 @@ newCredential :: BS.ByteString -- ^ value for oauth_token
               -> Credential
 newCredential tok sec = Credential [("oauth_token", tok), ("oauth_token_secret", sec)]
 
-token, tokenSecret :: Credential -> BS.ByteString
-token = fromMaybe "" . lookup "oauth_token" . unCredential
-tokenSecret = fromMaybe "" . lookup "oauth_token_secret" . unCredential
 
-data OAuthException = OAuthException String
-                      deriving (Show, Eq, Data, Typeable)
+-- | Empty credential.
+emptyCredential :: Credential
+emptyCredential = Credential []
 
-instance Exception OAuthException
 
-toStrict :: BSL.ByteString -> BS.ByteString
-toStrict = BS.concat . BSL.toChunks
+-- | Insert an oauth parameter into given 'Credential'.
+insert :: BS.ByteString -- ^ Parameter Name
+       -> BS.ByteString -- ^ Value
+       -> Credential    -- ^ Credential
+       -> Credential    -- ^ Result
+insert k v = Credential . insertMap k v . unCredential
+
+
+-- | Convenient method for inserting multiple parameters into credential.
+inserts :: [(BS.ByteString, BS.ByteString)] -> Credential -> Credential
+inserts = flip $ foldr (uncurry insert)
+
+
+-- | Remove an oauth parameter for key from given 'Credential'.
+delete :: BS.ByteString -- ^ Parameter name
+       -> Credential    -- ^ Credential
+       -> Credential    -- ^ Result
+delete key = Credential . deleteMap key . unCredential
+
+
+-- | Insert @oauth-verifier@ on a 'Credential'.
+injectVerifier :: BS.ByteString -> Credential -> Credential
+injectVerifier = insert "oauth_verifier"
+
+
+----------------------------------------------------------------------
+-- Signature
+
+-- | Add OAuth headers & sign to 'Request'.
+signOAuth :: MonadIO m
+          => OAuth              -- ^ OAuth Application
+          -> Credential         -- ^ Credential
+          -> Request            -- ^ Original Request
+          -> m Request          -- ^ Signed OAuth Request
+signOAuth oa crd req = signOAuth' oa crd addAuthHeader req
+
+-- | More flexible signOAuth
+signOAuth' :: MonadIO m
+          => OAuth              -- ^ OAuth Application
+          -> Credential         -- ^ Credential
+          -> (BS.ByteString -> Credential -> Request -> Request) -- ^ signature style
+          -> Request            -- ^ Original Request
+          -> m Request          -- ^ Signed OAuth Request
+signOAuth' oa crd add_auth req = do
+  crd' <- addTimeStamp =<< addNonce crd
+  let tok = injectOAuthToCred oa crd'
+  sign <- genSign oa tok req
+  return $ add_auth prefix (insert "oauth_signature" sign tok) req
+  where
+    prefix = case oauthRealm oa of
+      Nothing -> "OAuth "
+      Just v  -> "OAuth realm=\"" `BS.append` v `BS.append` "\","
+
+
+-- | Generate OAuth signature.  Used by 'signOAuth'.
+genSign :: MonadIO m => OAuth -> Credential -> Request -> m BS.ByteString
+genSign oa tok req =
+  case oauthSignatureMethod oa of
+    HMACSHA1 -> do
+      text <- getBaseString tok req
+      let key  = BS.intercalate "&" $ map paramEncode [oauthConsumerSecret oa, tokenSecret tok]
+      return $ encode $ toStrict $ bytestringDigest $ hmacSha1 (fromStrict key) text
+    PLAINTEXT ->
+      return $ BS.intercalate "&" $ map paramEncode [oauthConsumerSecret oa, tokenSecret tok]
+    RSASHA1 pr ->
+      liftM (encode . toStrict . rsassa_pkcs1_v1_5_sign hashSHA1 pr) (getBaseString tok req)
+
+
+----------------------------------------------------------------------
+-- Temporary credentails
 
-fromStrict :: BS.ByteString -> BSL.ByteString
-fromStrict = BSL.fromChunks . return
 
 -- | Get temporary credential for requesting acces token.
-getTemporaryCredential :: (MonadResource m, MonadBaseControl IO m)
+getTemporaryCredential :: MonadIO m
                        => OAuth         -- ^ OAuth Application
                        -> Manager
                        -> m Credential -- ^ Temporary Credential (Request Token & Secret).
 getTemporaryCredential = getTemporaryCredential' id
 
+
 -- | Get temporary credential for requesting access token with Scope parameter.
-getTemporaryCredentialWithScope :: (MonadResource m, MonadBaseControl IO m)
+getTemporaryCredentialWithScope :: MonadIO m
                                 => BS.ByteString -- ^ Scope parameter string
                                 -> OAuth         -- ^ OAuth Application
                                 -> Manager
                                 -> m Credential -- ^ Temporay Credential (Request Token & Secret).
 getTemporaryCredentialWithScope bs = getTemporaryCredential' (addScope bs)
 
-#if MIN_VERSION_http_conduit(2, 0, 0)
-addScope :: BS.ByteString -> Request -> Request
-#else
-addScope :: (MonadIO m) => BS.ByteString -> Request m -> Request m
-#endif
-addScope scope req | BS.null scope = req
-                   | otherwise     = urlEncodedBody [("scope", scope)] req
 
 -- | Get temporary credential for requesting access token via the proxy.
-getTemporaryCredentialProxy :: (MonadResource m, MonadBaseControl IO m)
+getTemporaryCredentialProxy :: MonadIO m
                             => Maybe Proxy   -- ^ Proxy
                             -> OAuth         -- ^ OAuth Application
                             -> Manager
                             -> m Credential -- ^ Temporary Credential (Request Token & Secret).
 getTemporaryCredentialProxy p oa m = getTemporaryCredential' (addMaybeProxy p) oa m
 
-getTemporaryCredential' :: (MonadResource m, MonadBaseControl IO m)
-#if MIN_VERSION_http_conduit(2, 0, 0)
+
+getTemporaryCredential' :: MonadIO m
                         => (Request -> Request)       -- ^ Request Hook
-#else
-                        => (Request m -> Request m)   -- ^ Request Hook
-#endif
                         -> OAuth                      -- ^ OAuth Application
                         -> Manager
                         -> m Credential    -- ^ Temporary Credential (Request Token & Secret).
@@ -189,19 +316,25 @@ getTemporaryCredential' hook oa manager = do
   let req = fromJust $ parseUrl $ oauthRequestUri oa
       crd = maybe id (insert "oauth_callback") (oauthCallback oa) $ emptyCredential
   req' <- signOAuth oa crd $ hook (req { method = "POST" })
-  rsp <- httpLbs req' manager
+  rsp <- liftIO $ httpLbs req' manager
   if responseStatus rsp == status200
     then do
       let dic = parseSimpleQuery . toStrict . responseBody $ rsp
       return $ Credential dic
     else liftIO . throwIO . OAuthException $ "Gaining OAuth Temporary Credential Failed: " ++ BSL.unpack (responseBody rsp)
 
+
+----------------------------------------------------------------------
+-- Authorization URL
+
+
 -- | URL to obtain OAuth verifier.
 authorizeUrl :: OAuth           -- ^ OAuth Application
              -> Credential      -- ^ Temporary Credential (Request Token & Secret)
              -> String          -- ^ URL to authorize
 authorizeUrl = authorizeUrl' $ \oa -> const [("oauth_consumer_key", oauthConsumerKey oa)]
 
+
 -- | Convert OAuth and Credential to URL to authorize.
 --   This takes function to choice parameter to pass to the server other than
 --   /oauth_callback/ or /oauth_token/.
@@ -217,18 +350,23 @@ authorizeUrl' f oa cr = oauthAuthorizeUri oa ++ BS.unpack (renderSimpleQuery Tru
             Just callback -> ("oauth_callback", callback):fixed
 
 
+----------------------------------------------------------------------
+-- Finishing authentication
+
+
 -- | Get Access token.
 getAccessToken, getTokenCredential
-               :: (MonadResource m, MonadBaseControl IO m)
+               :: MonadIO m
                => OAuth         -- ^ OAuth Application
                -> Credential    -- ^ Temporary Credential (with oauth_verifier if >= 1.0a)
                -> Manager
                -> m Credential -- ^ Token Credential (Access Token & Secret)
 getAccessToken = getAccessToken' id
 
+
 -- | Get Access token via the proxy.
 getAccessTokenProxy, getTokenCredentialProxy
-               :: (MonadResource m, MonadBaseControl IO m)
+               :: MonadIO m
                => Maybe Proxy   -- ^ Proxy
                -> OAuth         -- ^ OAuth Application
                -> Credential    -- ^ Temporary Credential (with oauth_verifier if >= 1.0a)
@@ -236,74 +374,40 @@ getAccessTokenProxy, getTokenCredentialProxy
                -> m Credential -- ^ Token Credential (Access Token & Secret)
 getAccessTokenProxy p = getAccessToken' $ addMaybeProxy p
 
-getAccessToken' :: (MonadResource m, MonadBaseControl IO m)
-#if MIN_VERSION_http_conduit(2, 0, 0)
+getAccessToken' :: MonadIO m
                 => (Request -> Request)       -- ^ Request Hook
-#else
-                => (Request m -> Request m)   -- ^ Request Hook
-#endif
                 -> OAuth                      -- ^ OAuth Application
                 -> Credential                 -- ^ Temporary Credential (with oauth_verifier if >= 1.0a)
                 -> Manager
                 -> m Credential     -- ^ Token Credential (Access Token & Secret)
-getAccessToken' hook oa cr manager = do
-  let req = hook (fromJust $ parseUrl $ oauthAccessTokenUri oa) { method = "POST" }
-  rsp <- flip httpLbs manager =<< signOAuth oa (if oauthVersion oa == OAuth10 then delete "oauth_verifier" cr else cr) req
-  if responseStatus rsp == status200
-    then do
-      let dic = parseSimpleQuery . toStrict . responseBody $ rsp
-      return $ Credential dic
-    else liftIO . throwIO . OAuthException $ "Gaining OAuth Token Credential Failed: " ++ BSL.unpack (responseBody rsp)
+getAccessToken' hook oauth cr manager = do
+    maybe_access_token <- getAccessTokenWith AccessTokenRequest { accessTokenAddAuth = addAuthHeader, accessTokenRequestHook = hook, accessTokenOAuth = oauth, accessTokenTemporaryCredential = cr, accessTokenManager = manager }
+    case maybe_access_token of 
+        Left error_response -> liftIO . throwIO . OAuthException $ "Gaining OAuth Token Credential Failed: " ++ BSL.unpack (responseBody error_response)
+        Right access_token -> return access_token
+
+getAccessTokenWith :: MonadIO m
+                => AccessTokenRequest -- ^ extensible parameters
+                -> m (Either (Response BSL.ByteString) Credential)     -- ^ Token Credential (Access Token & Secret) or the conduit response on failures
+getAccessTokenWith params = do
+      let req = hook (fromJust $ parseUrl $ oauthAccessTokenUri oa) { method = "POST" }
+      rsp <- liftIO $ flip httpLbs manager =<< signOAuth' oa (if oauthVersion oa == OAuth10 then delete "oauth_verifier" cr else cr) add_auth req
+      if responseStatus rsp == status200
+        then do
+          let dic = parseSimpleQuery . toStrict . responseBody $ rsp
+          return $ Right $ Credential dic
+        else
+          return $ Left rsp
+    where
+      add_auth = accessTokenAddAuth params
+      hook = accessTokenRequestHook params
+      oa = accessTokenOAuth params
+      cr = accessTokenTemporaryCredential params
+      manager = accessTokenManager params
 
 getTokenCredential = getAccessToken
 getTokenCredentialProxy = getAccessTokenProxy
 
-insertMap :: Eq a => a -> b -> [(a,b)] -> [(a,b)]
-insertMap key val = ((key,val):) . filter ((/=key).fst)
-
-deleteMap :: Eq a => a -> [(a,b)] -> [(a,b)]
-deleteMap k = filter ((/=k).fst)
-
--- | Insert an oauth parameter into given 'Credential'.
-insert :: BS.ByteString -- ^ Parameter Name
-       -> BS.ByteString -- ^ Value
-       -> Credential    -- ^ Credential
-       -> Credential    -- ^ Result
-insert k v = Credential . insertMap k v . unCredential
-
--- | Convenient method for inserting multiple parameters into credential.
-inserts :: [(BS.ByteString, BS.ByteString)] -> Credential -> Credential
-inserts = flip $ foldr (uncurry insert)
-
--- | Remove an oauth parameter for key from given 'Credential'.
-delete :: BS.ByteString -- ^ Parameter name
-       -> Credential    -- ^ Credential
-       -> Credential    -- ^ Result
-delete key = Credential . deleteMap key . unCredential
-
-injectVerifier :: BS.ByteString -> Credential -> Credential
-injectVerifier = insert "oauth_verifier"
-
--- | Add OAuth headers & sign to 'Request'.
-signOAuth :: (MonadUnsafeIO m)
-          => OAuth              -- ^ OAuth Application
-          -> Credential         -- ^ Credential
-#if MIN_VERSION_http_conduit(2, 0, 0)
-          -> Request            -- ^ Original Request
-          -> m Request          -- ^ Signed OAuth Request
-#else
-          -> Request m          -- ^ Original Request
-          -> m (Request m)    -- ^ Signed OAuth Request
-#endif
-signOAuth oa crd req = do
-  crd' <- addTimeStamp =<< addNonce crd
-  let tok = injectOAuthToCred oa crd'
-  sign <- genSign oa tok req
-  return $ addAuthHeader prefix (insert "oauth_signature" sign tok) req
-  where
-    prefix = case oauthRealm oa of
-      Nothing -> "OAuth "
-      Just v  -> "OAuth realm=\"" `BS.append` v `BS.append` "\","
 
 baseTime :: UTCTime
 baseTime = UTCTime day 0
@@ -315,14 +419,14 @@ showSigMtd PLAINTEXT = "PLAINTEXT"
 showSigMtd HMACSHA1  = "HMAC-SHA1"
 showSigMtd (RSASHA1 _) = "RSA-SHA1"
 
-addNonce :: MonadUnsafeIO m => Credential -> m Credential
+addNonce :: MonadIO m => Credential -> m Credential
 addNonce cred = do
-  nonce <- unsafeLiftIO $ replicateM 10 (randomRIO ('a','z')) -- FIXME very inefficient
+  nonce <- liftIO $ replicateM 10 (randomRIO ('a','z')) -- FIXME very inefficient
   return $ insert "oauth_nonce" (BS.pack nonce) cred
 
-addTimeStamp :: MonadUnsafeIO m => Credential -> m Credential
+addTimeStamp :: MonadIO m => Credential -> m Credential
 addTimeStamp cred = do
-  stamp <- (floor . (`diffUTCTime` baseTime)) `liftM` unsafeLiftIO getCurrentTime
+  stamp <- (floor . (`diffUTCTime` baseTime)) `liftM` liftIO getCurrentTime
   return $ insert "oauth_timestamp" (BS.pack $ show (stamp :: Integer)) cred
 
 injectOAuthToCred :: OAuth -> Credential -> Credential
@@ -332,47 +436,28 @@ injectOAuthToCred oa cred =
             , ("oauth_version", "1.0")
             ] cred
 
-#if MIN_VERSION_http_conduit(2, 0, 0)
-genSign :: MonadUnsafeIO m => OAuth -> Credential -> Request -> m BS.ByteString
-#else
-genSign :: MonadUnsafeIO m => OAuth -> Credential -> Request m -> m BS.ByteString
-#endif
-genSign oa tok req =
-  case oauthSignatureMethod oa of
-    HMACSHA1 -> do
-      text <- getBaseString tok req
-      let key  = BS.intercalate "&" $ map paramEncode [oauthConsumerSecret oa, tokenSecret tok]
-      return $ encode $ toStrict $ bytestringDigest $ hmacSha1 (fromStrict key) text
-    PLAINTEXT ->
-      return $ BS.intercalate "&" $ map paramEncode [oauthConsumerSecret oa, tokenSecret tok]
-    RSASHA1 pr ->
-      liftM (encode . toStrict . rsassa_pkcs1_v1_5_sign ha_SHA1 pr) (getBaseString tok req)
 
-#if MIN_VERSION_http_conduit(2, 0, 0)
+-- | Place the authentication information in a URL encoded body instead of the Authorization header.
+--
+-- Note that the first parameter is used for realm in addAuthHeader, and this
+-- function needs the same type. The parameter, however, is unused.
+--
+-- Since 1.5.1
+addAuthBody :: a -> Credential -> Request -> Request
+addAuthBody _ (Credential cred) req = urlEncodedBody (filterCreds cred) req
+
 addAuthHeader :: BS.ByteString -> Credential -> Request -> Request
-#else
-addAuthHeader :: BS.ByteString -> Credential -> Request a -> Request a
-#endif
 addAuthHeader prefix (Credential cred) req =
   req { requestHeaders = insertMap "Authorization" (renderAuthHeader prefix cred) $ requestHeaders req }
 
 renderAuthHeader :: BS.ByteString -> [(BS.ByteString, BS.ByteString)] -> BS.ByteString
-renderAuthHeader prefix = (prefix `BS.append`). BS.intercalate "," . map (\(a,b) -> BS.concat [paramEncode a, "=\"",  paramEncode b, "\""]) . filter ((`elem` ["oauth_token", "oauth_verifier", "oauth_consumer_key", "oauth_signature_method", "oauth_timestamp", "oauth_nonce", "oauth_version", "oauth_callback", "oauth_signature"]) . fst)
+renderAuthHeader prefix = (prefix `BS.append`). BS.intercalate "," . map (\(a,b) -> BS.concat [paramEncode a, "=\"",  paramEncode b, "\""]) . filterCreds
 
--- | Encode a string using the percent encoding method for OAuth.
-paramEncode :: BS.ByteString -> BS.ByteString
-paramEncode = BS.concatMap escape
-  where
-    escape c | isAscii c && (isAlpha c || isDigit c || c `elem` "-._~") = BS.singleton c
-             | otherwise = let num = map toUpper $ showHex (ord c) ""
-                               oct = '%' : replicate (2 - length num) '0' ++ num
-                           in BS.pack oct
+filterCreds :: [(BS.ByteString, BS.ByteString)] -> [(BS.ByteString, BS.ByteString)]
+filterCreds = filter ((`elem` ["oauth_token", "oauth_verifier", "oauth_consumer_key", "oauth_signature_method", "oauth_timestamp", "oauth_nonce", "oauth_version", "oauth_callback", "oauth_signature"]) . fst)
 
-#if MIN_VERSION_http_conduit(2, 0, 0)
-getBaseString :: MonadUnsafeIO m => Credential -> Request -> m BSL.ByteString
-#else
-getBaseString :: MonadUnsafeIO m => Credential -> Request m -> m BSL.ByteString
-#endif
+
+getBaseString :: MonadIO m => Credential -> Request -> m BSL.ByteString
 getBaseString tok req = do
   let bsMtd  = BS.map toUpper $ method req
       isHttps = secure req
@@ -382,31 +467,67 @@ getBaseString tok req = do
       bsURI = BS.concat [scheme, "://", host req, bsPort, path req]
       bsQuery = parseSimpleQuery $ queryString req
   bsBodyQ <- if isBodyFormEncoded $ requestHeaders req
-                  then liftM parseSimpleQuery $ toLBS (requestBody req)
+                  then liftM parseSimpleQuery $ toBS (requestBody req)
                   else return []
   let bsAuthParams = filter ((`elem`["oauth_consumer_key","oauth_token", "oauth_version","oauth_signature_method","oauth_timestamp", "oauth_nonce", "oauth_verifier", "oauth_version","oauth_callback"]).fst) $ unCredential tok
       allParams = bsQuery++bsBodyQ++bsAuthParams
-      bsParams = BS.intercalate "&" $ map (\(a,b)->BS.concat[a,"=",b]) $ sortBy compareTuple
+      bsParams = BS.intercalate "&" $ map (\(a,b)->BS.concat[a,"=",b]) $ sort
                    $ map (\(a,b) -> (paramEncode a,paramEncode b)) allParams
   -- parameter encoding method in OAuth is slight different from ordinary one.
   -- So this is OK.
   return $ BSL.intercalate "&" $ map (fromStrict.paramEncode) [bsMtd, bsURI, bsParams]
 
-#if MIN_VERSION_http_conduit(2, 0, 0)
-toLBS :: MonadUnsafeIO m => RequestBody -> m BS.ByteString
-toLBS (RequestBodyLBS l) = return $ toStrict l
-toLBS (RequestBodyBS s) = return s
-toLBS (RequestBodyBuilder _ b) = return $ toByteString b
-toLBS (RequestBodyStream _ givesPopper) = toLBS' givesPopper
-toLBS (RequestBodyStreamChunked givesPopper) = toLBS' givesPopper
 
-type Popper = IO BS.ByteString
-type NeedsPopper a = Popper -> IO a
-type GivesPopper a = NeedsPopper a -> IO a
+----------------------------------------------------------------------
+-- Utilities
 
-toLBS' :: MonadUnsafeIO m => GivesPopper () -> m BS.ByteString
--- FIXME probably shouldn't be using MonadUnsafeIO
-toLBS' gp = unsafeLiftIO $ do
+-- | Encode a string using the percent encoding method for OAuth.
+paramEncode :: BS.ByteString -> BS.ByteString
+paramEncode = BS.concatMap escape
+  where
+    escape c | isAscii c && (isAlpha c || isDigit c || c `elem` ("-._~" :: String)) = BS.singleton c
+             | otherwise = let num = map toUpper $ showHex (ord c) ""
+                               oct = '%' : replicate (2 - length num) '0' ++ num
+                           in BS.pack oct
+
+
+addScope :: BS.ByteString -> Request -> Request
+addScope scope req | BS.null scope = req
+                   | otherwise     = urlEncodedBody [("scope", scope)] req
+
+
+token, tokenSecret :: Credential -> BS.ByteString
+token = fromMaybe "" . lookup "oauth_token" . unCredential
+tokenSecret = fromMaybe "" . lookup "oauth_token_secret" . unCredential
+
+
+addMaybeProxy :: Maybe Proxy -> Request -> Request
+addMaybeProxy p req = req { proxy = p }
+
+
+insertMap :: Eq a => a -> b -> [(a,b)] -> [(a,b)]
+insertMap key val = ((key,val):) . filter ((/=key).fst)
+
+deleteMap :: Eq a => a -> [(a,b)] -> [(a,b)]
+deleteMap k = filter ((/=k).fst)
+
+
+toStrict :: BSL.ByteString -> BS.ByteString
+toStrict = BS.concat . BSL.toChunks
+
+fromStrict :: BS.ByteString -> BSL.ByteString
+fromStrict = BSL.fromChunks . return
+
+
+toBS :: MonadIO m => RequestBody -> m BS.ByteString
+toBS (RequestBodyLBS l) = return $ toStrict l
+toBS (RequestBodyBS s) = return s
+toBS (RequestBodyBuilder _ b) = return $ toByteString b
+toBS (RequestBodyStream _ givesPopper) = toBS' givesPopper
+toBS (RequestBodyStreamChunked givesPopper) = toBS' givesPopper
+
+toBS' :: MonadIO m => GivesPopper () -> m BS.ByteString
+toBS' gp = liftIO $ do
     ref <- I.newIORef BS.empty
     gp (go ref)
     I.readIORef ref
@@ -419,31 +540,7 @@ toLBS' gp = unsafeLiftIO $ do
             if BS.null bs
                 then I.writeIORef ref $ BS.concat $ front []
                 else loop (front . (bs:))
-#else
-toLBS :: MonadUnsafeIO m => RequestBody m -> m BS.ByteString
-toLBS (RequestBodyLBS l) = return $ toStrict l
-toLBS (RequestBodyBS s) = return s
-toLBS (RequestBodyBuilder _ b) = return $ toByteString b
-toLBS (RequestBodySource _ src) = toLBS' src
-toLBS (RequestBodySourceChunked src) = toLBS' src
 
-toLBS' :: MonadUnsafeIO m => Source m Builder -> m BS.ByteString
-toLBS' src = liftM BS.concat $ src $= builderToByteString $$ CL.consume
-#endif
 
 isBodyFormEncoded :: [Header] -> Bool
 isBodyFormEncoded = maybe False (=="application/x-www-form-urlencoded") . lookup "Content-Type"
-
-compareTuple :: (Ord a, Ord b) => (a, b) -> (a, b) -> Ordering
-compareTuple (a,b) (c,d) =
-  case compare a c of
-    LT -> LT
-    EQ -> compare b d
-    GT -> GT
-
-#if MIN_VERSION_http_conduit(2, 0, 0)
-addMaybeProxy :: Maybe Proxy -> Request -> Request
-#else
-addMaybeProxy :: Maybe Proxy -> Request m -> Request m
-#endif
-addMaybeProxy p req = req { proxy = p }
