@@ -5,16 +5,16 @@
 -- What this module do is just adding 'withManager' or 'runResourceT'.
 module Web.Authenticate.OAuth.IO
     {-# DEPRECATED "This module is deprecated; rewrite your code using MonadResource" #-}
-    ( 
+    (
       module Web.Authenticate.OAuth,
       getAccessToken,
       getTemporaryCredential, getTemporaryCredentialWithScope,
       getTemporaryCredentialProxy, getTemporaryCredential',
       getTokenCredential,
-      getAccessTokenProxy, getTokenCredentialProxy, 
+      getAccessTokenProxy, getTokenCredentialProxy,
       getAccessToken'
     ) where
-import Network.HTTP.Conduit
+import Network.HTTP.Client
 import qualified Web.Authenticate.OAuth as OA
 import Web.Authenticate.OAuth hiding
     (getAccessToken,
@@ -24,7 +24,6 @@ import Web.Authenticate.OAuth hiding
      getAccessTokenProxy, getTemporaryCredentialProxy,
      getTokenCredentialProxy,
      getAccessToken', getTemporaryCredential')
-import Data.Conduit
 import Control.Monad.IO.Class
 import qualified Data.ByteString.Char8 as BS
 
@@ -33,7 +32,7 @@ import qualified Data.ByteString.Char8 as BS
 getTemporaryCredential :: MonadIO m
                        => OA.OAuth        -- ^ OAuth Application
                        -> m OA.Credential -- ^ Temporary Credential (Request Token & Secret).
-getTemporaryCredential = liftIO . withManager . OA.getTemporaryCredential
+getTemporaryCredential = liftIO . withManager defaultManagerSettings . OA.getTemporaryCredential
 
 -- | Get temporary credential for requesting access token with Scope parameter.
 getTemporaryCredentialWithScope :: MonadIO m
@@ -41,7 +40,7 @@ getTemporaryCredentialWithScope :: MonadIO m
                                 -> OAuth         -- ^ OAuth Application
                                 -> m Credential -- ^ Temporay Credential (Request Token & Secret).
 getTemporaryCredentialWithScope bs oa =
-  liftIO $ withManager $ OA.getTemporaryCredentialWithScope bs oa
+  liftIO $ withManager defaultManagerSettings $ OA.getTemporaryCredentialWithScope bs oa
 
 
 -- | Get temporary credential for requesting access token via the proxy.
@@ -49,17 +48,13 @@ getTemporaryCredentialProxy :: MonadIO m
                             => Maybe Proxy   -- ^ Proxy
                             -> OAuth         -- ^ OAuth Application
                             -> m Credential -- ^ Temporary Credential (Request Token & Secret).
-getTemporaryCredentialProxy p oa = liftIO $ withManager $ OA.getTemporaryCredential' (addMaybeProxy p) oa
+getTemporaryCredentialProxy p oa = liftIO $ withManager defaultManagerSettings $ OA.getTemporaryCredential' (addMaybeProxy p) oa
 
 getTemporaryCredential' :: MonadIO m
-#if MIN_VERSION_http_conduit(2, 0, 0)
                         => (Request -> Request)                                 -- ^ Request Hook
-#else
-                        => (Request (ResourceT IO) -> Request (ResourceT IO))   -- ^ Request Hook
-#endif
                         -> OAuth                      -- ^ OAuth Application
                         -> m Credential -- ^ Temporary Credential (Request Token & Secret).
-getTemporaryCredential' hook oa = liftIO $ withManager $ OA.getTemporaryCredential' hook oa
+getTemporaryCredential' hook oa = liftIO $ withManager defaultManagerSettings $ OA.getTemporaryCredential' hook oa
 
 
 -- | Get Access token.
@@ -68,7 +63,7 @@ getAccessToken, getTokenCredential
                => OAuth         -- ^ OAuth Application
                -> Credential    -- ^ Temporary Credential with oauth_verifier
                -> m Credential -- ^ Token Credential (Access Token & Secret)
-getAccessToken oa cr = liftIO $ withManager $ OA.getAccessToken oa cr
+getAccessToken oa cr = liftIO $ withManager defaultManagerSettings $ OA.getAccessToken oa cr
 
 -- | Get Access token via the proxy.
 getAccessTokenProxy, getTokenCredentialProxy
@@ -77,18 +72,14 @@ getAccessTokenProxy, getTokenCredentialProxy
                -> OAuth         -- ^ OAuth Application
                -> Credential    -- ^ Temporary Credential with oauth_verifier
                -> m Credential -- ^ Token Credential (Access Token & Secret)
-getAccessTokenProxy p oa cr = liftIO $ withManager $ OA.getAccessTokenProxy p oa cr
+getAccessTokenProxy p oa cr = liftIO $ withManager defaultManagerSettings $ OA.getAccessTokenProxy p oa cr
 
 getAccessToken' :: MonadIO m
-#if MIN_VERSION_http_conduit(2, 0, 0)
                 => (Request -> Request)                                 -- ^ Request Hook
-#else
-                => (Request (ResourceT IO) -> Request (ResourceT IO))   -- ^ Request Hook
-#endif
                 -> OAuth                      -- ^ OAuth Application
                 -> Credential                 -- ^ Temporary Credential with oauth_verifier
                 -> m Credential     -- ^ Token Credential (Access Token & Secret)
-getAccessToken' hook oa cr = liftIO $ withManager $ OA.getAccessToken' hook oa cr
+getAccessToken' hook oa cr = liftIO $ withManager defaultManagerSettings $ OA.getAccessToken' hook oa cr
 
 
 getTokenCredential = getAccessToken
